#!/usr/bin/python
# -*- coding: utf-8 -*-

import gpib
import serial
import time
import datetime
from time import gmtime, strftime

def query(handle, command, numbytes=100):
        gpib.write(handle,command)
        time.sleep(0.1)
        response = gpib.read(handle,numbytes)
        response = response.rstrip("\r\n") 
        return response
        
def command(handle, command):
        gpib.write(handle,command)
        time.sleep(0.1)

def gread(handle, numbytes=100):
        response = gpib.read(handle,numbytes)
        time.sleep(0.1)        
        response = response.rstrip("\r\n") 
        return response        
        
def tread(sensor):
    	ser.write(sensor)
    	time.sleep(0.5)  #give the serial port sometime to receive the data
        data = ser.readline().rstrip() # read data from serial
        if data == "125.0000":
            print "*** Error Reading ***"
        return data
            	
csvfilename = "dmm-"+strftime("%Y-%m-%d.%H.%M.%S")+".csv"
csvfile = open(csvfilename,'at')	# append,text mode

csvdelim = ';'		# CSV delimiter
meastime = 5		# Sec between print on screen 
logevery = 12		# Every 'th measurement should go itnto the logfile
        
HP3457= gpib.find("HP3457A")            # defined in /etc/gpib.conf - HP SVR-T
K2015= gpib.find("KEITHLEY2015")        # defined in /etc/gpib.conf - K SVR
gpib.clear(HP3457)
gpib.clear(K2015)

command(HP3457,"END ALWAYS")	# Force EOI on end of data (Default off)
 
print query(HP3457,"ID?")
print query(K2015,"*IDN?")
command(HP3457,"DCV3,,;NPLC 100;TRIG HOLD")				# DCvolt 3v range , 100NPLC (max) , Manual TRIG

command(K2015,":Sens:Func 'Volt:DC'")					# DCvolt 
command(K2015,":SENS:VOLT:DC:AVER:STAT OFF")			# No statistics (def. 10 pt moving average)
command(K2015,":SENSe:VOLTage:DC:RANGe 10:NPLC 10") 	# DCvolt 10v range , 10NPLC (max)

        
ser = serial.Serial(
     port='/dev/ttyACM0',
     baudrate=9600,
     bytesize=serial.EIGHTBITS,
     parity=serial.PARITY_NONE,
     stopbits=serial.STOPBITS_ONE,
     #timeout=0.1,
     #timeout=1,
     #xonxoff=0,
     #rtscts=0,
     #dtr=0,
     #interCharTimeout=None
 )

time.sleep(3)	 # Give the AVR some tme to startup
ser.flushInput() #flush input buffer, discarding all its contents
ser.flushOutput()#flush output buffer, aborting current output 

time.sleep(2)
ser.write("$r")
time.sleep(0.5)
ser.write("$S00,005")
time.sleep(0.5)
#ser.write("$p")
#time.sleep(0.5)

ser.flushInput() #flush input buffer, discarding all its contents
ser.flushOutput()#flush output buffer, aborting current output

readings = 0

while True:  
#    print "-"
    time.sleep(meastime - 5) 					# Compensate the time it takes to make the readings

#
# Trigger readings on both meters
#
    command(HP3457,"TRIG SGL") 					# Trigger one reading
    command(K2015,":SENSE:DATA:FRESH?") 		# Takes ? sec
    time.sleep(2)								# Wait xNPLC time (100 = 2 sec)
#
# Begin read HP        
#
    str_hptemp = tread("$t1")	 				# Read temp on sensor1 (HP) SVR-T on HP
#    command(HP3457,"TRIG SGL") 					# Trigger one reading
#    time.sleep(2)								# Wait xNPLC time (100 = 2 sec)

    str_hpvolt = gread(HP3457)	 				# Get voltage , HP returns a leading space 
    str_hpvolt = str_hpvolt.strip()  			# remove leading and trailing spaces if any
    
    str_hphires = query(HP3457,"RMATH HIRES")	# Get the extra digit in HIRES - 
    											# Example = -1.5375000E-06  (HP automaticaly scales it so it can be added directly
    											
#    str_hphires = str_hphires.strip()			# remove leading and trailing spaces if any
    										
    f_hpvolt = float(str_hpvolt)				# Convert ascii to float		
    f_hphires = float(str_hphires)				# Convert ascii to float
    f_hp7digit = f_hpvolt+f_hphires				# Add hires digit to voltage (The HP automaticly scales it to correct value)
    str_hp7digit = str(f_hp7digit)				# Convert 7.7 digit to string
    
#
# End read HP        
#

#
# Begin read K2015      
#
    str_ktemp = tread("$t2")						# Read temp on sensor1 (K2015) SVR on Keithley
#    kvolt = query(K2015,":READ?") # Takes ? sec
#    kvolt = query(K2015,":FETCH?") # Takes ? sec 	# ** Watch out read doc on FETCH   
#    command(K2015,":SENSE:DATA:FRESH?") 			# Takes ? sec   
    str_kvolt = gread(K2015)	 					# Get voltage , K returns a leading '+'
    str_kvolt = str_kvolt.strip()					# remove leading and trailing spaces if any

#
# End read K2015      
#

#
# Build print line
#    

#    print strftime("%Y-%m-%d"),csvdelim,strftime("%H:%M:%S"),csvdelim,hptemp,csvdelim,str_hpvolt,csvdelim,ktemp,csvdelim,kvolt,csvdelim
#   measline =  strftime("%Y-%m-%d")+csvdelim+strftime("%H:%M:%S")+csvdelim+hptemp+csvdelim+str_hpvolt+csvdelim+ktemp+csvdelim+kvolt+csvdelim    

#   measline    Collums : Date;Time;HP-Temp;HP-Volt;HP-Highres;HP-7.5digit;K2015-Temp;K2015-Volt;
    measline =  strftime("%Y-%m-%d")+csvdelim+strftime("%H:%M:%S")+csvdelim
    measline =  measline+str_hptemp+csvdelim+str_hpvolt+csvdelim+str_hphires+csvdelim+str_hp7digit+csvdelim
    measline =  measline+str_ktemp+csvdelim+str_kvolt+csvdelim    
    print measline
    readings = readings + 1
#   Line looks like this : 2014-01-09;20:54:16;24.0625;9.9999800E+00;23.8750;+9.99995872E+00;

#
# Log to file if needed
#
    if readings == logevery:
    	readings = 0
    	csvfile.write(measline+'\n')
    	csvfile.flush()
    	

    	
    
    
    

    
    
